local Sudoku = LibStub('AceAddon-3.0'):NewAddon('Sudoku', 'AceTimer-3.0', 'AceEvent-3.0', 'AceConsole-3.0')
_G.Sudoku = Sudoku

local SML = LibStub("LibSharedMedia-3.0")
local defaults = {
	global = {
		gameLog = {},
		stepsPerFrame = 75,
		blankCells = 55,
		hideInCombat = true,
		announceNewRecords = true,
		puzzleBoard = nil,
		completeBoard = nil,
		savedTime = 0
	}
}
local db

local function GetDB()
	if not db then
		db = LibStub('AceDB-3.0'):New('SudokuDB', defaults, 'Default')
	end
	return db["global"]
end

function Sudoku:SaveFrameLocation(frameName,frame)
	local p1, _, p2, x, y = frame:GetPoint()
	GetDB()[frameName] = {}
	GetDB()[frameName]["p1"] = p1
	GetDB()[frameName]["p2"] = p2
	GetDB()[frameName]["x"] = x
	GetDB()[frameName]["y"] = y
end

function Sudoku:SetFrameLocation(frameName,frame)
	local p1, p2, x, y = "Center", "Center",0,0
	if GetDB()[frameName] then
		p1 = GetDB()[frameName]["p1"]
		p2 = GetDB()[frameName]["p2"]
		x = GetDB()[frameName]["x"]
		y = GetDB()[frameName]["y"]
	end
	frame:SetPoint(p1,frame:GetParent(),p2,x,y)
end

function Sudoku:LogGame(isWin, duration, isLegit)
	tinsert(GetDB()["gameLog"], {isWin,duration,isLegit,blankCells})
end

function Sudoku:GetGameLog()
	return GetDB()["gameLog"]
end

function Sudoku:ClearGameLog()
	GetDB()["gameLog"] = {}
	print("History Cleared")
end

function Sudoku:GetBestGame()
	if #(GetDB()["gameLog"]) == 0 then
		return nil
	end
	local best = {false,99999,false}
	for i,game in ipairs(GetDB()["gameLog"]) do 
		if game[1] == true and (best[1] == false or (game[3] == true and best[3] == false) or game[2] < best[2]) then
			best = game
		end
	end
	return best
end

function Sudoku:PrintGameLog()
	local text = ""
	if #(GetDB()["gameLog"]) == 0 then
		print("Empty History")
	end
	for i,game in ipairs(GetDB()["gameLog"]) do 
		if game[1] == false then
			text = "Loss"
		elseif game[2] < 60 then
			text = "Win " .. (format('%d seconds',game[2]))
		elseif game[2] < 3600 then
			text = "Win " .. (format('%d minutes %02d seconds', 1/60 * game[2], game[2] % 60))
		elseif game[2] < 86400 then
			text = "Win " .. (format('%d hours %02d minutes', 1/3600 * game[2], 1/60 * (game[2] % 3600)))
		else
			text = "Win " .. (format('%d days %02d hours', 1/86400 * game[2], 1/3600 * (game[2] % 86400)))		
		end
		if game[3] == true then
			print(i..":",text)
		else
			print(i..":","*Cheated*",text)
		end
	end
end

function Sudoku:GetOptionValue(info)
	return GetDB()[info.arg]
end

function Sudoku:SetOptionValue(info, value)
	GetDB()[info.arg] = value
end

function Sudoku:GetStepsPerFrame()
	return GetDB()["stepsPerFrame"]
end

function Sudoku:GetBlankCells()
	return GetDB()["blankCells"]
end

function Sudoku:GetHideInCombat()
	return GetDB()["hideInCombat"]
end

function Sudoku:GetAnnounceNewRecords()
	return GetDB()["announceNewRecords"]
end

function Sudoku:HasSavedGame()
	return (GetDB()["puzzleBoard"] ~= nil and GetDB()["completeBoard"] ~= nil)
end

function Sudoku:SetSavedGame(puzzleBoard,completeBoard,time)
	GetDB()["puzzleBoard"] = puzzleBoard
	GetDB()["completeBoard"] = completeBoard
	GetDB()["savedTime"] = time
end

function Sudoku:GetSavedGame()
	return GetDB()["puzzleBoard"],GetDB()["completeBoard"],GetDB()["savedTime"]
end